//--------------------------------------------------------------------------------
// dual_paraboloid_environmentmap_terrain.fx
//
//--------------------------------------------------------------------------------


//-----------------------------------------------------------------------------
float4x4	mWVP		: WorldViewProjection;	// world * view matrix ( projection is identity )
float4		viewParms	: ViewParm0;			// ***.z = 1 for forward hemisphere, = -1 for backward hemisphere
//-----------------------------------------------------------------------------
texture		texture0	: ColorMap0;	// mud.dds
texture		texture1	: ColorMap1;	// grass.dds
//-----------------------------------------------------------------------------
sampler Sampler0 = sampler_state
{
    Texture   = (texture0);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler Sampler1 = sampler_state
{
    Texture   = (texture1);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
struct vertex
{
    float3 position	 : POSITION;
    float4 color     : COLOR0;
    float2 texcoord0  : TEXCOORD0;
    float2 texcoord1  : TEXCOORD1;
};
//-----------------------------------------------------------------------------
struct fragment
{
    float4 position   : POSITION;
	float4 color	  : COLOR0;
	float2 tex0coord  : TEXCOORD0;
    float2 tex1coord  : TEXCOORD1;
    float  z_value    : TEXCOORD2;
};
//-----------------------------------------------------------------------------
struct pixel
{
	float4 color : COLOR;
};
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Vertex Shader
//-----------------------------------------------------------------------------
fragment VS( vertex IN )
{
    fragment OUT;

	OUT.color		= IN.color;			// pass through the vertex color
	OUT.tex0coord	= IN.texcoord0;		// pass through texcoords set 0
	OUT.tex1coord	= IN.texcoord1;		// pass through texcoords set 1

	float direction = viewParms.z;		// this determines the front or back hemisphere

	OUT.position = mul( float4(IN.position, 1), mWVP );	// transform vertex into the maps basis
	OUT.position = OUT.position / OUT.position.w;		// divide by w to normalize

	OUT.position.z = OUT.position.z * direction;		// set z-values to forward or backward
	
	float L = length( OUT.position.xyz );				// determine the distance between (0,0,0) and the vertex
	OUT.position = OUT.position / L;					// divide the vertex position by the distance 
	
	OUT.z_value = OUT.position.z;						// remember which hemisphere the vertex is in
	OUT.position.z = OUT.position.z + 1;				// add the reflected vector to find the normal vector

	OUT.position.x = OUT.position.x / OUT.position.z;	// divide x coord by the new z-value
	OUT.position.y = OUT.position.y / OUT.position.z;	// divide y coord by the new z-value

	OUT.position.z = L / 500;							// set a depth value for correct z-buffering
	OUT.position.w = 1;									// set w to 1 so there is no w divide

	return OUT;
}
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Pixel shaders
//-----------------------------------------------------------------------------
pixel PS( fragment IN )
{
    pixel OUT;

	clip( IN.z_value );

	float4 tex0 = tex2D( Sampler0, IN.tex0coord ) * IN.color.r;
	float4 tex1 = tex2D( Sampler1, IN.tex1coord ) * (IN.color.g + IN.color.b);
	
	OUT.color = (tex0 + tex1);

    return OUT;
}
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
technique TwoPassTextureBlend
{
    pass P0
    {
		//FillMode = Wireframe;
	
		CullMode = NONE;
   	
   		VertexShader = compile vs_1_1 VS();
		PixelShader  = compile ps_2_0 PS();
    }
}
//-----------------------------------------------------------------------------
